package writecache

import (
	"fmt"
	"io"
	"os"

	common "git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-lens/internal"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/writecache/writecachebadger"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/writecache/writecachebbolt"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/util/logger"
	oid "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/object/id"
	"github.com/spf13/cobra"
)

var listCMD = &cobra.Command{
	Use:   "inspect",
	Short: "Object inspection",
	Long:  `Inspect specific object in a write-cache.`,
	Run:   listFunc,
}

func init() {
	common.AddComponentPathFlag(listCMD, &vPath)
}

func listFunc(cmd *cobra.Command, _ []string) {
	// other targets can be supported
	w := cmd.OutOrStderr()

	wAddr := func(addr oid.Address) error {
		_, err := io.WriteString(w, fmt.Sprintf("%s\n", addr))
		return err
	}

	switch vDBType {
	case "bbolt":
		db, err := writecachebbolt.OpenDB(vPath, true, os.OpenFile)
		common.ExitOnErr(cmd, common.Errf("could not open write-cache db: %w", err))
		defer db.Close()

		err = writecachebbolt.IterateDB(db, wAddr)
		common.ExitOnErr(cmd, common.Errf("write-cache iterator failure: %w", err))

	case "badger":
		log, err := logger.NewLogger(&logger.Prm{})
		common.ExitOnErr(cmd, common.Errf("could not create logger: %w", err))

		db, err := writecachebadger.OpenDB(vPath, true, log)
		common.ExitOnErr(cmd, common.Errf("could not open write-cache db: %w", err))

		err = writecachebadger.IterateDB(db, wAddr)
		common.ExitOnErr(cmd, common.Errf("write-cache iterator failure: %w", err))

	default:
		common.ExitOnErr(cmd, fmt.Errorf("invalid dbtype: %q (possible values: bbolt, badger)", vDBType))
	}
}
