package profilerconfig_test

import (
	"testing"
	"time"

	"git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-node/config"
	profilerconfig "git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-node/config/profiler"
	configtest "git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-node/config/test"
	"github.com/stretchr/testify/require"
)

func TestProfilerSection(t *testing.T) {
	t.Run("defaults", func(t *testing.T) {
		to := profilerconfig.ShutdownTimeout(configtest.EmptyConfig())
		addr := profilerconfig.Address(configtest.EmptyConfig())

		require.Equal(t, profilerconfig.ShutdownTimeoutDefault, to)
		require.Equal(t, profilerconfig.AddressDefault, addr)
		require.False(t, profilerconfig.Enabled(configtest.EmptyConfig()))

		require.Zero(t, profilerconfig.BlockRate(configtest.EmptyConfig()))
		require.Zero(t, profilerconfig.MutexRate(configtest.EmptyConfig()))
	})

	const path = "../../../../config/example/node"

	fileConfigTest := func(c *config.Config) {
		to := profilerconfig.ShutdownTimeout(c)
		addr := profilerconfig.Address(c)

		require.Equal(t, 15*time.Second, to)
		require.Equal(t, "localhost:6060", addr)
		require.True(t, profilerconfig.Enabled(c))

		require.Equal(t, 10_000, profilerconfig.BlockRate(c))
		require.Equal(t, 10_000, profilerconfig.MutexRate(c))
	}

	configtest.ForEachFileType(path, fileConfigTest)

	t.Run("ENV", func(t *testing.T) {
		configtest.ForEnvFileType(t, path, fileConfigTest)
	})
}
