package shard

import (
	"context"
	"testing"
	"time"

	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/core/object"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/internal/testutil"
	"git.frostfs.info/TrueCloudLab/frostfs-sdk-go/client"
	cidtest "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/container/id/test"
	"github.com/stretchr/testify/require"
)

func TestShard_Delete(t *testing.T) {
	t.Parallel()

	t.Run("without write cache", func(t *testing.T) {
		t.Parallel()
		testShardDelete(t, false)
	})

	t.Run("with write cache", func(t *testing.T) {
		t.Parallel()
		testShardDelete(t, true)
	})
}

func testShardDelete(t *testing.T, hasWriteCache bool) {
	sh := newShard(t, hasWriteCache)

	cnr := cidtest.ID()

	obj := testutil.GenerateObjectWithCID(cnr)
	testutil.AddAttribute(obj, "foo", "bar")

	var putPrm PutPrm
	var getPrm GetPrm

	t.Run("big object", func(t *testing.T) {
		testutil.AddPayload(obj, 1<<20)

		putPrm.SetObject(obj)
		getPrm.SetAddress(object.AddressOf(obj))

		var delPrm DeletePrm
		delPrm.SetAddresses(object.AddressOf(obj))

		_, err := sh.Put(context.Background(), putPrm)
		require.NoError(t, err)

		_, err = testGet(t, sh, getPrm, hasWriteCache)
		require.NoError(t, err)

		if hasWriteCache {
			require.Eventually(t, func() bool {
				_, err = sh.Delete(context.Background(), delPrm)
				return err == nil
			}, 30*time.Second, 100*time.Millisecond)
		} else {
			_, err = sh.Delete(context.Background(), delPrm)
			require.NoError(t, err)
		}

		_, err = sh.Get(context.Background(), getPrm)
		require.True(t, client.IsErrObjectNotFound(err))
	})

	t.Run("small object", func(t *testing.T) {
		obj := testutil.GenerateObjectWithCID(cnr)
		testutil.AddAttribute(obj, "foo", "bar")
		testutil.AddPayload(obj, 1<<5)

		putPrm.SetObject(obj)
		getPrm.SetAddress(object.AddressOf(obj))

		var delPrm DeletePrm
		delPrm.SetAddresses(object.AddressOf(obj))

		_, err := sh.Put(context.Background(), putPrm)
		require.NoError(t, err)

		_, err = sh.Get(context.Background(), getPrm)
		require.NoError(t, err)

		if hasWriteCache {
			require.Eventually(t, func() bool {
				_, err = sh.Delete(context.Background(), delPrm)
				return err == nil
			}, 10*time.Second, 100*time.Millisecond)
		} else {
			_, err = sh.Delete(context.Background(), delPrm)
			require.NoError(t, err)
		}

		_, err = sh.Get(context.Background(), getPrm)
		require.True(t, client.IsErrObjectNotFound(err))
	})
}
