package frostfs

import (
	"fmt"

	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/morph/client"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/morph/event"
	"github.com/nspcc-dev/neo-go/pkg/core/state"
	"github.com/nspcc-dev/neo-go/pkg/util"
)

type Config struct {
	KeyValue   []byte
	ValueValue []byte
	IDValue    []byte

	// TxHashValue is used in notary environmental
	// for calculating unique but same for
	// all notification receivers values.
	TxHashValue util.Uint256
}

// TxHash returns hash of the TX with new epoch
// notification.
func (u Config) TxHash() util.Uint256 {
	return u.TxHashValue
}

// MorphEvent implements Neo:Morph Event interface.
func (Config) MorphEvent() {}

func (u Config) ID() []byte { return u.IDValue }

func (u Config) Key() []byte { return u.KeyValue }

func (u Config) Value() []byte { return u.ValueValue }

func ParseConfig(e *state.ContainedNotificationEvent) (event.Event, error) {
	var (
		ev  Config
		err error
	)

	params, err := event.ParseStackArray(e)
	if err != nil {
		return nil, fmt.Errorf("could not parse stack items from notify event: %w", err)
	}

	if ln := len(params); ln != 3 {
		return nil, event.WrongNumberOfParameters(3, ln)
	}

	// parse id
	ev.IDValue, err = client.BytesFromStackItem(params[0])
	if err != nil {
		return nil, fmt.Errorf("could not get config update id: %w", err)
	}

	// parse key
	ev.KeyValue, err = client.BytesFromStackItem(params[1])
	if err != nil {
		return nil, fmt.Errorf("could not get config key: %w", err)
	}

	// parse value
	ev.ValueValue, err = client.BytesFromStackItem(params[2])
	if err != nil {
		return nil, fmt.Errorf("could not get config value: %w", err)
	}

	ev.TxHashValue = e.Container

	return ev, nil
}
