package deletesvc

import (
	"context"

	"git.frostfs.info/TrueCloudLab/frostfs-node/internal/logs"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/services/object/util"
	"go.uber.org/zap"
)

// Delete serves requests to remote the objects.
func (s *Service) Delete(ctx context.Context, prm Prm) error {
	// If session token is not found we will fail during tombstone PUT.
	// Here we fail immediately to ensure no unnecessary network communication is done.
	if tok := prm.common.SessionToken(); tok != nil {
		_, err := s.keyStorage.GetKey(&util.SessionInfo{
			ID:    tok.ID(),
			Owner: tok.Issuer(),
		})
		if err != nil {
			return err
		}
	}

	exec := &execCtx{
		svc: s,
		prm: prm,
	}

	exec.setLogger(s.log)

	return exec.execute(ctx)
}

func (exec *execCtx) execute(ctx context.Context) error {
	exec.log.Debug(logs.ServingRequest)

	if err := exec.executeLocal(ctx); err != nil {
		exec.log.Debug(logs.OperationFinishedWithError, zap.String("error", err.Error()))
		return err
	}

	exec.log.Debug(logs.OperationFinishedSuccessfully)
	return nil
}
